// Ionic Starter App

// angular.module is a global place for creating, registering and retrieving Angular modules
// 'starter' is the name of this angular module example (also set in a <body> attribute in index.html)
// the 2nd parameter is an array of 'requires'
// 'starter.controllers' is found in controllers.js
angular.module('app',
    ['ionic', 'ngSanitize', 'pascalprecht.translate', 'ui.bootstrap',
     'app.controllers', 'app.services', 'app.routes', 'app.directives', 'angucomplete-alt'],
    function ($httpProvider) {
        $httpProvider.defaults.headers.post['Content-Type'] = 'application/x-www-form-urlencoded;charset=utf-8'; // Use x-www-form-urlencoded Content-Type
        
        $httpProvider.defaults.timeout = window.CONFIG.HTTP_TIMEOUT;

        /**
        * The workhorse; converts an object to x-www-form-urlencoded serialization.
        * @param {Object} obj
        * @return {String}
        */
        var param = function (obj) {


        var query = '', name, value, fullSubName, subName, subValue, innerObj, i;

        for (name in obj) {
          value = obj[name];

          if (value instanceof Array) {
            for (i=0; i < value.length; ++i) {
              subValue = value[i];
              fullSubName = name + '[' + i + ']';
              innerObj = {};
              innerObj[fullSubName] = subValue;
              query += param(innerObj) + '&';
            }
          } else if (value instanceof Object) {
            for (subName in value) {
              subValue = value[subName];
              fullSubName = name + '[' + subName + ']';
              innerObj = {};
              innerObj[fullSubName] = subValue;
              query += param(innerObj) + '&';
            }
          } else if (value !== undefined && value !== null){
            query += encodeURIComponent(name) + '=' + encodeURIComponent(value) + '&';
		  }
        }

        console.log (">> query "+ query);

        return query.length ? query.substr(0, query.length - 1) : query;
    };

    // Override $http service's default transformRequest
    $httpProvider.defaults.transformRequest = [function(data) {
        return angular.isObject(data) && String(data) !== '[object File]' ? param(data) : data;
        }];
    })

    .run(function ($ionicPlatform, $ionicPopup, $state, $rootScope, $ionicViewService, $ionicBackdrop, $http, $timeout, $translate, $locale, ProfileService, BiometricService, NotificationsService) {
        $ionicPlatform.ready(function () {
            selectCurrentLanguage($translate);

	        $locale.DATETIME_FORMATS.shortDate = 'dd/MM/yyyy';

            //$rootScope.developmentMode = CONFIG.WAREABLE_TEST;
            cordova.getAppVersion.getVersionNumber().then(function (version) {
                CONFIG.VERSION = version;
                $rootScope.versionNumber = version;
            });

            //imposto l'ambiente corrente
            setEnv();
            $rootScope.ambiente = getEnv();

            if(window.Keyboard) {
              // Hide the accessory bar by default (remove this to show the accessory bar above the keyboard
              // for form inputs)
              window.Keyboard.hideFormAccessoryBar(false);

              // Don't remove this line unless you know what you are doing. It stops the viewport
              // from snapping when text inputs are focused. Ionic handles this internally for
              // a much nicer keyboard experience.
              // window.Keyboard.disableScroll(true);
            }

            testPlugins();

            if (window.StatusBar) {
                window.StatusBar.styleLightContent();
                window.StatusBar.show();
            }

            document.addEventListener("pause", onPause, false);
            document.addEventListener("resume", onResume, false);
            document.addEventListener('deviceready', function () {
                cordova.plugins.notification.local.fireQueuedEvents();
            }, false);

            // cordova.plugins.backgroundMode.on('activate', function() {
            //     cordova.plugins.backgroundMode.disableWebViewOptimizations();
            //     cordova.plugins.backgroundMode.moveToBackground();
            // });

            cordova.plugins.notification.local.on("click", function (notification, state) {
                $rootScope.$broadcast('notificationClicked',{ notification: notification, state: state });
            }, $rootScope);

            cordova.plugins.IsDebug.getIsDebug(function(isDebug) {
                if (!isDebug){
                    //override console.log for production
                    var console = {};
                    console.log = function(){};
                    console.info = function(){};
                    console.error = function(){};
                    console.warn = function(){};
                    window.console = console;
                }
            }, function(err) {
                console.error(err);
            });


            /** compatibilità ionic */
            window.skipLocalNotificationReady = true;


            /** background task configuration */
            (function(){
                console.log("setting up background tasks");
                /** setting up background tasks */
                var BackgroundFetch = window.BackgroundFetch;

                // Your background-fetch handler.
                var fetchCallback = function() {
                    console.log('BackgroundFetch event received');

                    performBackgroundTasks();

                    // Required: Signal completion of your task to native code
                    // If you fail to do this, the OS can terminate your app
                    // or assign battery-blame for consuming too much background-time

                    setTimeout(function(){
                        BackgroundFetch.finish()
                    }, 100);
                };

                var failureCallback = function(error) {
                    console.log('- BackgroundFetch failed', error);
                };

                BackgroundFetch.configure(fetchCallback, failureCallback, {
                    minimumFetchInterval: 15, // <-- default is 15
                    stopOnTerminate: false,   // <-- Android only
                    startOnBoot: true,        // <-- Android only
                    forceReload: false         // <-- Android only
                });
            })();

            $rootScope.isTablet = isTablet();
            $rootScope.biometricType = '';
                             
            if (window.plugins.biometrictype){
                window.plugins.biometrictype.biometricType(
                    function (res){
                        $rootScope.biometricType = res;
                    }
                )}
            
            //google analytics
            if (window.analytics) window.analytics.startTrackerWithId(CONFIG.analyticsTrackerId);

            //push setup
            pushMeApp.init($http, $ionicPopup, CONFIG.VERSION);

            function onPause(){
                if (CONFIG.HOLD_APP_RESUME){
                    console.log("pausing paused!");
                    return;
                }
                console.log("pausing..." + $state.current.name);
                $rootScope.$broadcast('onPause');
                //reset contatore notifiche
                cordova.plugins.notification.badge.clear();
                saveState($state);
                store('lastAccess', new Date().getTime());
                persistLocalStorage();
                closeAllModals();
            }

            function onResume() {

                if (CONFIG.HOLD_APP_RESUME){
                    console.log("resuming paused!");
                    return;
                }
                console.log("resuming... ");
                selectCurrentLanguage($translate);
                $rootScope.$broadcast('onResume');

                //se impostata l'autenticazione tramite touch-id faccio scarede il login dopo un'ora
                if (window.plugins && window.plugins.touchid && BiometricService.getBiometricLoginProtectionConfig()) {
                    var lastAccess = retrieve('lastAccess');
                    //se passata un'ora richiedo il login
                    if (lastAccess && (new Date().getTime() - parseInt(lastAccess) > CONFIG.biometricLoginExpirationPeriod)) {
                        $rootScope.partialLogout();
                    }
                }

                checkRemoteUpdates();

                var profile = getCurrentUser();
                if (profile.authToken) {

                    retrieveProfiledStoredData(profile);

                    ProfileService.checkUser().then(function (result) {
                        if (result) {
                            profile = result;

                            if (profile.authError != null) {
                                $ionicPopup.alert({
                                    template: profile.authError,
                                    title: $translate.instant("alert-title-attenzione")
                                }).then(function () {
                                    $rootScope.logout();
                                });
                            }
                            else {
                                storeCurrentUser(profile);
                                console.log("currentUser", profile);

                                $rootScope.currentUser = profile;
                                var loggedUser = getLoggedUser();

                                if (null === loggedUser){
                                    loggedUser = profile;
                                    $rootScope.loggedUser = loggedUser;
                                    storeLoggedUser(loggedUser);
                                }
                                console.log("loggedUser", loggedUser);

                                //primo accesso: forzo il cambio password
                                if (profile.totaleCambiPassword == 0 && !loggedUser.abilitatoCambioAzienda){

                                    $ionicPopup.alert({
                                        template: $translate.instant("alert-template-primo-accesso"),
                                        title: $translate.instant("alert-title-cambio-password")
                                    }).then(function () {
                                        $rootScope.goto("cambio-password");
                                    });

                                }
                                else if (profile.abilitatoPrivacy && !profile.privacyAccepted){

                                    $ionicPopup.alert({template: profile.flagPrivacyConsenso === null ? $translate.instant("alert-template-privacy-mancante") : $translate.instant("alert-template-privacy"),
                                        title: $translate.instant("alert-title-attenzione")}).then(function(){
                                        $rootScope.goto("privacy");
                                    });
                                }
                                else if (!
                                        ($state.current.name === "generacodice-carte" ||
                                    $state.current.name === "storicocodici-carte" ||
                                    $state.current.name === "visualizza-pin" ||
                                    $state.current.name === "dpayment-erogazione" ||
                                    $state.current.name === "loyalty")){

                                    $state.go("private-dashboard");
                                }
                            }
                        }
                    })
                }
                else {
                    if ($state.current.name !== "login"){
                    $state.go('public-dashboard');
                }
                }

            }

            //se non impostato registro l'alias
            var profile = getCurrentUser();

            if (profile.authToken != null) {
                pushMeApp.getAlias(profile.email).then(function (result) {
                    if (result.responseStatus != null) {
                        console.log("Getting Alias error [" + result.responseStatus + "] " + result.responseMessage)
                    } else {
                        if (result.alias != profile.email) {
                            pushMeApp.setAlias(profile.email);
                        }
                    }
                })
            }            

            // try {
            //     wearable.initialize({ httpHandle: $http });
            //
            // } catch(err) {
            //     console.log("ERRORE " + err);
            // }

            onResume();
            //dpay sdk initialization
            initSdk();

            /**
             * permette di cambiare ambiente da produzione a collaudo
             */
            $rootScope.countClickForContextSwitch = function(){
                $rootScope.clickCounter = $rootScope.clickCounter || 1;
                if ($rootScope.clickCounter++ % 7 === 0){
                    var context = retrieve("ambiente");

                    switch(context){
                        case 'local':
                            context = "coll";
                            break;
                        case 'coll':
                            context = "prod";
                            break;
                        case 'prod':
                            context = "local";
                            break;
                    }

                    $rootScope.ambiente = context;

                    setEnv($rootScope.ambiente);
                    $ionicPopup.alert({
                        title: 'CAMBIO AMBIENTE',
                        template: "Il nuovo ambiente è " + context
                    }).then(function(){
                        $rootScope.clickCounter = 1;
                    })
                }
            }

            /**
             * @param keyEvent: $event
             * @param elementId: l'id dell'elemento su cui chiamare il focus
             */
            $rootScope.focus = function(keyEvent, elementId) {
                if (keyEvent.keyCode === 13) {
                    var el = document.getElementById(elementId) || document.getElementsByName(elementId)[0];
                    el.focus();
                }
            };

        $rootScope.registerModal = function(modal){
            $rootScope.modalStack = $rootScope.modalStack || [];
            $rootScope.modalStack.push(modal);
        };

        var closeAllModals = function(){
            //chiusura modali
            if ($rootScope.modalStack){
                for (var i=0; i<$rootScope.modalStack.length; i++){
                    $rootScope.modalStack[i].hide();
                }
            }
            $rootScope.modalStack = [];

            //chiusura popup
            var noop = angular.noop;
            var popupStack = $ionicPopup._popupStack;
            if (popupStack.length > 0) {
                popupStack.forEach(function(popup, index) {
                    if (popup.isShown === true) {
                        popup.remove();
                        popupStack.pop();
                    }
                });
            }

            $ionicBackdrop.release();
            //Remove popup-open & backdrop if this is last popup
            $timeout(function() {
                // wait to remove this due to a 300ms delay native
                // click which would trigging whatever was underneath this
                document.querySelector('body').classList.remove('popup-open')

            }, 400, false);
            ($ionicPopup._backButtonActionDone || noop)();
        }

            function performBackgroundTasks (){
                console.log("performing background tasks", new Date());

                if (profile && profile.authToken) {
                    //verifica presenza nuove notizie
                    cordova.plugins.notification.badge.get(function (badge) {
                        console.log(">>> badge " + badge, badge);
                        if (badge === 0) {
                            console.log(">>> verifico presenza di nuove comunicazioni...");
                            NotificationsService.checkNuoveComunicazioni().then(function (response) {
                                console.log(">>> checkNuoveComunicazioni " + response.value, response);
                                if (response.value && response.errorMessage === null) {
                                    if (response.value !== 0 && badge !== response.value) {
                                        console.log(">>> sending local notification");
                                        cordova.plugins.notification.local.schedule({
                                            title: 'Avviso',
                                            text: 'Hai un nuovo messaggio',
                                            foreground: true
                                        });
                                    }
                                    cordova.plugins.notification.badge.set(response.value);
                                }
                            }).catch(function (error) {
                                console.error("checkNuoveComunicazioni", error);
                            })
                        }
                    });
                }
            }

            function checkRemoteUpdates() {
                var key = (isAndroid() ? CONFIG.context.ANDROID_CODE_PUSH_KEY : CONFIG.context.IOS_CODE_PUSH_KEY);
                codePush.sync(null, { deploymentKey: key, installMode: InstallMode.IMMEDIATE});
            }

            // //wearable
            // try {
            //     wearable.initialize({ httpHandle: $http });
            // } catch(err) {
            //     console.log("ERRORE " + err);
            // }

        });

        $ionicPlatform.registerBackButtonAction(function (event) {
            if($state.current.name=="public-dashboard" || $state.current.name=="private-dashboard"){
                $ionicPopup.show({
                    template: $translate.instant("alert-template-uscire-applicazione"),
                    title: $translate.instant("alert-title-attenzione"),
                    buttons: [
                        {
                            text: $translate.instant("alert-btn-annulla"),
                            type: 'button-default'
                        },
                        {
                            text: $translate.instant("alert-btn-ok"),
                            type: 'button-positive',
                            onTap: function(e) {
                                navigator.app.exitApp();
                            }
                        }
                    ]
                });
            }
            else {
                var view = $ionicViewService.getBackView();
                if (view) view.go();
            }
        }, 100);

        $rootScope.openStoreLocator = function(){
            var url = CONFIG.context.storeLocatorUrl;
            var profile = retrieve("profile");
            if (profile && profile.authToken){
                url += "/cartissima/"+profile.authToken;
            }
            console.log("opening Store locator " + url);
            var ref = cordova.InAppBrowser.open(url, '_system', 'location=no, clearcache=yes');
        }

        $rootScope.openUrl = function(url){
            var ref = cordova.InAppBrowser.open(url, '_system', 'location=no, clearcache=yes');
        }

        function initSdk() {
            console.log("DPaySdk [" + DPaySDK.Environment[getEnv().toUpperCase()] + "]: "+ CONFIG.context.DPAY_SDK_APP_KEY);
            DPaySDK.initializeSdk(CONFIG.context.DPAY_SDK_APP_KEY, DPaySDK.Environment[getEnv().toUpperCase()], CONFIG.DPAY_CARD_TYPE,
                function(success){
                    console.log("DPaySdk completed", success);
                },
                function(error){
                    console.error("initializeSdk", error);
                });
        }

    })

	.filter('unsafe', function($sce) {
		return $sce.trustAsHtml;
	})

	.filter("htmlSafe", ['$sce', function($sce) {
	    return function(htmlCode){
	        return $sce.trustAsHtml(htmlCode);
	    };
	}])
;

