angular.module('app.controllers')

    .controller('LoyaltyCtrl', function($state, $scope, $rootScope, $ionicPopup, $ionicLoading, $ionicModal, $ionicViewService, $translate, LoyaltyService) {
        console.log("LoyaltyCtrl");
        trackView( $state.current.name );

        //oggetto per le condidioni generali
        $scope.tc = {};
        $scope.data = {tcConfim1: null};

        // var callback;

        $ionicModal.fromTemplateUrl(
            'templates/loyalty/privacy.html',
            {
                scope: $scope,
                animation: 'slide-in-up'
            }
        ).then(function (popover) {
            $scope.modal = popover;
            $rootScope.registerModal(popover);
        });

        $scope.$on('$stateChangeSuccess', function(ev){
            checkPrivacy();
        });

        $scope.openPrivacyModal = function() {
            $scope.modal.show();
        };
        $scope.closePrivacyModal = function() {
            $scope.modal.hide();
            $scope.data = { tcConfim1: null };
        };
        $scope.discardPrivacyModal = function() {
            $scope.closePrivacyModal();
            $ionicViewService.getBackView().go();
        };
        // Cleanup the modal when we're done with it!
        $scope.$on('$destroy', function() {
            $scope.modal.remove();
            $scope.data = { tcConfim1: null };
        });

        $scope.scaricaCondizioni = function(){
            console.log("scaricaCondizioni", $scope.tc.filename);
            window.open($scope.tc.filename ,'_system', 'location=no');
        };

        /**
         * controlla la sottoscrizione del contratto e prosegue l'azione richiesta
         * @param handler
         */
        function checkPrivacy() {
            // callback = handler; //salvo il riferimento nel caso la privacy non sia impostata

            $ionicLoading.show();
            LoyaltyService.checkPrivacy().then(function(response){
                $ionicLoading.hide();
                console.log("openLoyaltyCatalog", response);
                if (null !== response.errorMessage){
                    $ionicPopup.alert({template: response.errorMessage,
                        title: $translate.instant("alert-title-attenzione")});
                } else {
                    $scope.tc = response.value;
                    if (!response.value.agreed) {
                        $scope.openPrivacyModal();
                    } else {
                        console.log("calling handler");
                        if (handler) handler();
                    }
                }
            })

        }

        /**
         * verifica le condizioni contrattuali e rimanda alla pagina del catalogo
         */
        $scope.openLoyaltyCatalog = function(){
            // checkPrivacy( function(){
                window.open($scope.tc.catalogUrl ,'_system', 'location=no');
            // } );

        };

        /**
         * verifica le condizioni contrattuali e rimanda al wallet punti
         */
        $scope.gotoLoyaltyWallet = function(){
            // checkPrivacy( function(){
                $state.transitionTo('loyalty-wallet');
            // } );
        };

        /**
         * verifica le condizioni contrattuali e rimanda alla lista premi
         */
        $scope.gotoLoyaltyReward = function(){
            // checkPrivacy( function(){
                $state.transitionTo('loyalty-reward');
            // } );
        };

        /**
         * conferma la privacy e prosegue l'azione impostata in precedenza
         * @param formData
         */
        $scope.privacyConfirm = function(formData){

            console.log("privacyConfirm ", formData);

            //todo: check formData

            $ionicLoading.show();
            LoyaltyService.confirmPrivacy(formData).then(function(response){
                $ionicLoading.hide();

                if (null !== response.errorMessage){
                    $ionicPopup.alert({template: response.errorMessage,
                        title: $translate.instant("alert-title-attenzione")});
                } else {
                    console.log("calling handler");
                    $scope.closePrivacyModal();
                    // if (callback) callback();
                }
            })


        }

    })

    .controller('LoyaltyWalletCtrl', function($state, $scope, $rootScope, $ionicPopup, $ionicLoading, $ionicModal, $translate, LoyaltyService) {
        console.log("LoyaltyWalletCtrl");
        trackView( $state.current.name );

        var pageCount = 0;
        $scope.hasMoreData = false;
        $scope.showNoResultsFoundMessage = false;
        $scope.result = {};

        function loadWallet() {
            $ionicLoading.show();
            LoyaltyService.loadWallet(pageCount).then(function (response) {
                $ionicLoading.hide();

                console.log("loadWallet", response);

                if (null !== response.errorMessage) {
                    $ionicPopup.alert({template: response.errorMessage,
                        title: $translate.instant("alert-title-attenzione")});
                } else {

                    if (0 === pageCount){
                        $scope.result = response.value;
                    } else {

                        for (var i = 0; i < response.value.historyList.length; i++) {
                            $scope.result.historyList.push(response.value.historyList[i]);
                            $scope.$broadcast('scroll.infiniteScrollComplete');
                        }
                    }
                    pageCount++;
                    $scope.hasMoreData = response.value.historyList.length > 0;
                    console.log("$scope.hasMoreData", $scope.hasMoreData);
                }
            });
        }

        $scope.loadMore = function() {
            console.log("loadMore", $scope.hasMoreData);
            if ($scope.hasMoreData) {
                loadWallet()
            }
        };

        loadWallet();
    })

    .controller('LoyaltyRewardCtrl', function($state, $scope, $rootScope, $ionicPopup, $ionicLoading, $ionicModal, $translate, LoyaltyService) {
        console.log("LoyaltyRewardCtrl");
        trackView( $state.current.name );

        var pageCount = 0;
        $scope.hasMoreData = false;
        $scope.showNoResultsFoundMessage = true;
        $scope.results = {};

        function loadRewards() {
            $ionicLoading.show();
            LoyaltyService.loadRewards(pageCount).then(function (response) {
                $ionicLoading.hide();

                console.log("loadRewards", response);

                if (null !== response.errorMessage) {
                    $ionicPopup.alert({template: response.errorMessage,
                        title: $translate.instant("alert-title-attenzione")});
                } else {

                    if (0 === pageCount){
                        $scope.results = response.value;
                    } else {

                        for (var i = 0; i < response.value.length; i++) {
                            $scope.results.push(response.value[i]);
                            $scope.$broadcast('scroll.infiniteScrollComplete');
                        }
                    }
                    pageCount++;
                    $scope.hasMoreData = response.value.length > 0;
                    console.log("$scope.hasMoreData", $scope.hasMoreData);
                }
            });
        }

        $scope.loadMore = function() {
            console.log("loadMore", $scope.hasMoreData);
            if ($scope.hasMoreData) {
                loadRewards()
            }
        };

        loadRewards();
    })

;