angular.module('app.services')

    .service('LoginService', function($http, $ionicPopup, $translate) {
        return {
            login: function(user, pass) {

                if (undefined === window.device) {
                    window.device = {
                        "uuid": '12345',
                        "platform": 'web',
                        "version": '1',
                        "model": 'chrome'
                    };
                }
                var oPayload = {"jusr": user, "jpwd": pass};
                var auth = jwtToken(oPayload);

                return $http.post(CONFIG.apiUrl + '/login', {
                    // "jusr": user,
                    // "jpwd": pass,
                    "auth": auth,
                    "udid": window.device.uuid,
                    "platform": window.device.platform,
                    "version": window.device.version,
                    "model": window.device.model,
                    "appversion": CONFIG.VERSION
                }).success(function (response) {
                    console.log("login - success: " + JSON.stringify(response));
                    if (response.errorMessage !== null) {
                        return response.data;
                    }
                }).error(function(response){
                    console.log("login - error: " + JSON.stringify(response));
                    // $ionicPopup.alert({template: $translate.instant("html.cartissima.generacodice.error.code.99"),
                    //     title: $translate.instant("alert-title-attenzione")});
                });
            }

            //todo: logout
            ,
            forgotPasswordRequest: function(username) {
                return $http.post(CONFIG.apiUrl + '/public/pwdReset', {
                        email: username}
                ).success(function (response) {
                        if (response.errorMessage != null) {
                            console.log(response.errorMessage);
                        }
                    });
            },
            forgotPasswordConfirm: function(token, iscrData) {
                return $http.post(CONFIG.apiUrl + '/pwdResetConfirm',
                    {
                        token: token,
                        nuovaPwd: iscrData.nuovaPwd,
                        confermaPwd: iscrData.confermaPwd
                    }
                ).success(function (response) {
                        if (response.errorMessage != null) {
                            console.log(response.errorMessage);
                        }
                    });
            },
            logAs: function(codice) {
                var profile = getCurrentUser();
                return $http.post(CONFIG.apiUrl + '/logAs',
                    {
                        token: profile.authToken,
                        main: codice
                    }
                ).success(function (response) {
                        if (response.errorMessage != null) {
                            console.log(response.errorMessage);
                        }
                });
            }
        }
    })

    .service('ProfileService', function($http) {
        return {
            retrieveIndirizziConsegna: function() {
                var profile = getCurrentUser();
                return $http.get(CONFIG.apiUrl + "/profile/indirizziConsegna",
                    {params: {"token": profile.authToken}}).then(function (response) {
                        if (response.data) {
                            return response.data;
                        }
                    });
            },
            changePassword: function(iscrData) {
                var profile = getCurrentUser();

                return $http.post(CONFIG.apiUrl + '/password',
                    {
                        token: profile.authToken,
                        vecchiaPwd: iscrData.vecchiaPwd,
                        nuovaPwd: iscrData.nuovaPwd,
                        confermaPwd: iscrData.confermaPwd
                    }
                ).success(function (response) {
                        if (response.errorMessage != null) {
                            console.log(response.errorMessage);
                        }
                    });
            },
            clienti: function() {
                var profile = getCurrentUser();

                return $http.get(CONFIG.apiUrl + "/profile/clienti",
                  {params: {"token": profile.authToken}}).then(function (response) {
                  if (response.data) {
                    return response.data;
                  }
                });
            },
            checkUser: function() {
                var profile = getCurrentUser();
                
                return $http.get(CONFIG.apiUrl + "/profile/checkuser",{
                    params: {
                        "token": profile.authToken,
                        "udid": window.device ? window.device.uuid : 'browser',
                        "platform": window.device ? window.device.platform : 'browser',
                        "version": window.device ? window.device.version : '1.0',
                        "model": window.device ? window.device.model : 'browser',
                        "appversion": CONFIG.VERSION

                    }}).then(function (result) {
                        if (result.data.authError == null) {
                            store('profile', result.data);
                        }

                        return result.data;
                });
            },
            refreshToken: function() {
                var profile = getCurrentUser();

                return $http.get(CONFIG.apiUrl + "/profile/refreshtokens",{
                    params: {
                        "token": profile.authToken
                    }}).then(function (result) {
                        if (result.data.authError == null) {
                            profile.authToken = result.data.authToken;
                            profile.sdkToken = result.data.sdkToken;
                            store('profile', result.data);
                        }

                        return result.data;
                });
            },
            getPrivacy: function() {
                var profile = getCurrentUser();
                return $http.get(CONFIG.apiUrl + "/profile/privacy",{
                    params: {
                        "token": profile.authToken
                    }},{timeout: CONFIG.HTTP_TIMEOUT}).then(
                    function (response) {
                        return response.data;
                    }, function (error) {
                        console.log("ERROR getPrivacy " + JSON.stringify(error));
                        return '';
                    });
            },
            changePrivacy: function(data) {
                var profile = getCurrentUser();

                return $http.post(CONFIG.apiUrl + '/profile/privacy',
                    {
                        token: profile.authToken,
                        flagPrivacyConsenso: data.privacyFlag1,
                        flagPrivacyMarketing: data.privacyFlag2,
                        flagPrivacyProfilazione: data.privacyFlag3,
                        flagPrivacyTerzeParti: data.privacyFlag4
                    }
                ).success(function (response) {
                    if (response.errorMessage != null) {
                        console.log(response.errorMessage);
                    } else {
                        store('profile', response.value);
                    }
                });
            },
            listaMain: function(codiceMain){
                var loggedUser = getLoggedUser();
                return $http.post(CONFIG.apiUrl + '/listamain',
                    {
                        token: loggedUser.authToken,
                        main: codiceMain
                    }
                );
            },
            logAs: function(codiceMain, idAccount){
                var loggedUser = getLoggedUser();
                return $http.post(CONFIG.apiUrl + '/logAs',
                    {
                        token: loggedUser.authToken,
                        main: codiceMain,
                        id: idAccount
                    }
                );
            }
        }
    })

    .service('NotificationsService', function($http, $translate) {

        this.checkNuoveComunicazioni = function(){

            var profile = getLoggedUser();

            return $http.get(CONFIG.apiUrl + "/communications/check",
                { params: { "token": profile.authToken }},
                { timeout: CONFIG.HTTP_TIMEOUT }).then(
                function (response) {
                    return response.data;
                }, function(error){
                    console.log("ERROR checkNuoveComunicazioni " + JSON.stringify(error));
                    return {
                        errorMessage: $translate.instant("error.service.not.available")
                    }
                });
        };

        this.listNuoveNotifiche = function(){

            var profile = getLoggedUser();

            return $http.get(CONFIG.apiUrl + "/notifiche/unread",
                { params: { "token": profile.authToken }},
                { timeout: CONFIG.HTTP_TIMEOUT }).then(
                function (response) {
                    return response.data;
                }, function(error){
                    console.log("ERROR listNuoveNotifiche " + JSON.stringify(error));
                    return {
                        errorMessage: $translate.instant("error.service.not.available")
                    }
                });
        };


        this.markNewsAsRead = function(notifica){

            var profile = getLoggedUser();

            return $http.post(CONFIG.apiUrl + "/notifiche/read",
                { token: profile.authToken,
                    id: notifica.id },
                { timeout: CONFIG.HTTP_TIMEOUT }).then(
                function (response) {
                    return response.data;
                }, function(error){
                    console.log("ERROR changePassword " + JSON.stringify(error));
                    return {
                        errorMessage: $translate.instant("error.service.not.available")
                    }
                });
        };
    })

    .service('AccountService', function($http, $ionicLoading, $state) {
        return {
            ricercaByMain: function() {

                $ionicLoading.show();

                var profile = getCurrentUser();

                return $http.post(CONFIG.apiUrl + '/accounts',
                    {
                        token: profile.authToken,
                        main: profile.codiceMain
                    }
                ).success(function (response) {
                        $ionicLoading.hide();
                        if (response.errorMessage != null) {
                            console.log(response.errorMessage);
                        }
                    }).error(function(data, status, headers, config){
                        $ionicLoading.hide();
                        if (status == '401') {
                            // Unauthorized
                            $state.go('login');
                        } else {
                    // $ionicPopup.alert({template: $translate.instant("html.cartissima.generacodice.error.code.99"),
                    //     title: $translate.instant("alert-title-attenzione")});
                    }
                });
            }
        }
    })
;

