angular.module('app.services')
    .service('SpeechService', function($http) {

    	var REC_AUDIO = 'mic.wav';
        var recAudio = null;
	    var recInProgress = false;
	    var phoneType = getPlatform();

	    var _onSuccess; //riferimenti per android
	    var _onError;   //riferimenti per android

    	this.startRecording = function (durata, onSuccess, onError) {
		    console.log("Start audio recording");

		    var self = this;

		    if (phoneType === 'android') {
		        _onSuccess = onSuccess;
                _onError = onError;
                recAudio = new Media(REC_AUDIO, elaborateAudio, onError);
            } else
			    recAudio = new Media(REC_AUDIO, null, onError);

		    recAudio.startRecord();

		    recInProgress = true;

		    setTimeout(function() {
                if (recInProgress) {
                    recInProgress = false;
                    self.stopRecording(onSuccess, onError);
                }
            }, durata);
	    };


	    this.stopRecording = function (onSuccess, onError) {
		    recAudio.stopRecord();
		    recAudio.release();
		    recInProgress = false;
            if (phoneType === 'ios') elaborateAudio(onSuccess, onError);
        };

	    this.release = function () {
		    recAudio.release();
        };


	    function elaborateAudio(onSuccess, onError) {
            var path;

            if (phoneType === 'android') {
                path = cordova.file.externalRootDirectory + REC_AUDIO;
                window.requestFileSystem(LocalFileSystem.PERSISTENT, 0, gotFS, fail);
            } else {
                path = cordova.file.tempDirectory + REC_AUDIO;
                window.requestFileSystem(LocalFileSystem.TEMPORARY, 0, gotFS, fail);
            }

            console.log("Elaborating audio file " + path);


            function gotFS(fileSystem) {
                console.log("Got FileSystem");

                window.resolveLocalFileSystemURL(
                    path,
                    function (fileEntry) {
                        console.log("Got FileEntry with resolveLocalFileSystemURL");
                        gotFileEntry(fileEntry);
                    },
                    function (error) {
                        console.log("Error #" + error.code + " with resolveLocalFileSystemURL");
                    }
                )
            }

            function gotFileEntry(fileEntry) {
                console.log("Got FileEntry");
                fileEntry.file(gotFile, fail);
            }
            function fail(error) {
                console.log("Error #" + error.code);
            }
            function gotFile(file){
                console.log("Got File");
                readDataUrl(file);
            }

            function readDataUrl(file) {
                console.log("Reading data Url...");
                var reader = new FileReader();

                reader.onloadend = function(evt) {
                    console.log("Read as data URL", evt.target.result);

                    speechToText(
                        evt.target.result,  //send audio base64
                        function (response) {
                            if (response == undefined || response.length == 0) {
                                return;
                            }

                            console.log("Speech text: ", response);

	                        if (onSuccess) onSuccess(response);
	                        else
	                            _onSuccess(response);
                        },
                        function (error) {
							if (onError) onError(error);
							else
                                _onError(error);
                        }
                    );
                };

                reader.readAsDataURL(file);
            }
        }


        function speechToText(audioBase64, onSuccess, onError) {

	        var postData = { audioBase64: audioBase64 };

            return $http({
                method: 'POST',
                url: CONFIG.context.sdkUrl+ "/api/public/speechToText",
                headers: {
                    'Content-Type': 'application/json'
                },
                data: JSON.stringify(postData),
                timeout: 5000
            })
            .success(
                function (response) {
                    console.log("speechToText", response);
	                onSuccess(response);
                }
            ).error(
	            function(error) {
                    console.error("ERROR speechToText", error);
                    onError(error);
                }
            );
        }


		function getPlatform() {
			var phoneType = 'unknown';
		    var userAgent = navigator.userAgent || navigator.vendor || window.opera;

			if (userAgent.match( /iPad/i ) || userAgent.match( /iPhone/i ) || userAgent.match( /iPod/i ) )
			   phoneType = 'ios';
			else if( userAgent.match( /Android/i ) )
			   phoneType = 'android';

			return phoneType;
		}
    })
;
