/**
 * pushmeapp.js v 1.3.3
 */
(function (global) {
	"use strict";

  var config = {
    serverUrl: function(){
      var PUSH_SERVER_URL = 'http://pushmeapp.net/api/push';
      return PUSH_SERVER_URL;
    },
    apiKey: function(){
      var APP_KEY = '08591054-3a6f-4047-8d0b-54c3df9df502';
      return APP_KEY;
    }
  }

  var pushMeApp =  {

    _http: null,
    _popup: null,
    device:{},

    test: function() {
      console.log("test");
    },
    init: function($http, $popup, appVersion) {
      appVersion = appVersion || '';
      console.log("initializing push " + appVersion);
      var self = this;
      self._http = $http;
      self._popup = $popup;
      self.device = global.device || { udid: '123456-7890', platform: 'browser', manufacturer: 'Chrome', model: 'xyz', version: '1.0' };

      //registrazione Firebase
      if (global.FirebasePlugin) global.FirebasePlugin.grantPermission();  //popup di richiesta permessi push

      if (global.FirebasePlugin) global.FirebasePlugin.onTokenRefresh(function(token) {
        // save this server-side and use it to push notifications to this device
        console.log("FirebasePlugin onTokenRefresh " + token);
        if (token) registerToken(token);
      }, function(error) {
        console.error("FirebasePlugin " + error);
      });

      if (global.FirebasePlugin) global.FirebasePlugin.onNotificationOpen(function (notification) {

        //app aperta
        if (notification.tap === false) {
          var message =  '';
          if (self.device.platform == 'iOS') {
            message = notification.aps.alert.body;
          } else {
            //android {"tap":false,"body":"benvenuto in Q8easy CLUB","title":"Q8easy CLUB produzione"}
            message = notification.body;
          }
          if (self._popup){
            self._popup.alert({template: message,
              title: "Attenzione"});
          }
        } else {
          //app chiusa
          //app open on push
          // android {"google.sent_time":1511186215086,"tap":true,"from":"431599341284","google.message_id":"0:1511186215109411%9af209039af20903","collapse_key":"it.archeometra.q8.easy"}
          // ios {"aps":{"alert":{"title":"Q8easy CLUB","body":"benvenuto in Q8easy CLUB"},"sound":"default"},"gcm.message_id":"0:1511194274953859%9af209039af20903","tap":true}
          if (notification['google.message_id']) notifyAppOpened(notification['google.message_id']);
          if (notification['gcm.message_id']) notifyAppOpened(notification['gcm.message_id']);
        }

      }, function (error) {
        console.error("FirebasePlugin " + error);
      });

      function registerToken(token){
        // if (_http) {

        self._http({
            url: config.serverUrl() + "/register_device/" + config.apiKey(),
            method: "POST",
            params: {
              token: token,
              udid: self.device.uuid,
              platform: self.device.platform,
              manufacturer: self.device.manufacturer,
              model: self.device.model,
              version: self.device.version,
              app_version: appVersion
            }
          }).then(function successCallback(response) {
            console.log("PUSH registerToken " + JSON.stringify(response));
          }, function errorCallback(response) {
            console.log("ERROR PUSH registerToken " + JSON.stringify(response));
          });

        // } else {
        //   var params = {
        //     token: token,
        //     udid: device.uuid,
        //     platform: device.platform,
        //     manufacturer: device.manufacturer,
        //     model: device.model,
        //     version: device.version,
        //     app_version: appVersion
        //   };
        //
        //   $.ajax({
        //     url: PUSH_SERVER_URL + "/register_device/" + APP_KEY,
        //     type: "POST",
        //     // dataType: 'json',
        //     data: params,
        //     success: function (response) {
        //       console.log("PUSH registration" + JSON.stringify(response));
        //     },
        //     error: function (jqXHR, textStatus, errorThrown) {
        //       console.log("ERROR PUSH registration " + JSON.stringify(errorThrown));
        //     }
        //   });
        // }
      }

      function notifyAppOpened(messageId){
        this._http({
          url: config.serverUrl()+"/received/"+config.apiKey(),
          method: "POST",
          params: {
            messageId: messageId
          }
        }).then(function successCallback(response) {
          console.log("PUSH notifyAppOpened " + JSON.stringify(response));
          return response.data;
        }, function errorCallback(response) {
          console.log("ERROR PUSH notifyAppOpened " + JSON.stringify(response));
          return {responseStatus: 'timeout', response: response }
        });

      }

    },

    getAlias: function(){
      return this._http({
        url: config.serverUrl()+"/device_alias/"+config.apiKey(),
        method: "GET",
        params: {
          udid: this.device.uuid
        }
      }).then(function successCallback(response) {
        console.log("PUSH getAlias " + JSON.stringify(response));
        return response.data;
      }, function errorCallback(response) {
        console.log("ERROR PUSH getAlias " + JSON.stringify(response));
        return {responseStatus: 'timeout', response: response }
      });
    },

    setAlias: function(alias){
      return this._http({
        url: config.serverUrl()+"/device_alias/"+config.apiKey(),
        method: "POST",
        params: {
          alias: alias,
          udid: this.device.uuid
        }
      }).then(function successCallback(response) {
        console.log("PUSH setAlias " + JSON.stringify(response));
        return response.data;
      }, function errorCallback(response) {
        console.log("ERROR PUSH setAlias " + JSON.stringify(response));
        return {responseStatus: 'timeout', response: response }
      });

    },

    updateAlias: function(alias){
      var self = this;
      self.getAlias(alias).then(function (result) {
        if (result.responseStatus != null) {
          console.log("ERROR updateAlias [" + result.responseStatus + "] " + result.responseMessage)
        } else {
          if (result.alias != alias) {
            self.setAlias(alias);
          }
        }
      })
    },

    setUserNotificationsEnabled: function(enabled){
      return this._http({
        url: config.serverUrl()+"/device_user_notifications_enabled/"+config.apiKey(),
        method: "POST",
        params: {
          enabled: enabled,
          udid: this.device.uuid
        }
      }).then(function successCallback(response) {
        console.log("PUSH setUserNotificationsEnabled " + JSON.stringify(response));
        return response.data;
      }, function errorCallback(response) {
        console.log("ERROR PUSH setUserNotificationsEnabled " + JSON.stringify(response));
        return {responseStatus: 'timeout', response: response }
      });

    },

    setQuietTimeEnabled: function(enabled){
      return this._http({
        url: config.serverUrl()+"/device_quiet_time_enabled/"+config.apiKey(),
        method: "POST",
        params: {
          enabled: enabled,
          udid: this.device.uuid
        }
      }).then(function successCallback(response) {
        console.log("PUSH setQuietTimeEnabled " + JSON.stringify(response));
        return response.data;
      }, function errorCallback(response) {
        console.log("ERROR PUSH setQuietTimeEnabled " + JSON.stringify(response));
        return {responseStatus: 'timeout', response: response }
      });

    },

    setQuietTime: function(startHour, startMinute, endHour, endMinute){
      return this._http({
        url: config.serverUrl()+"/device_quiet_time/"+config.apiKey(),
        method: "POST",
        params: {
          startHour: startHour,
          startMinute: startMinute,
          endHour: endHour,
          endMinute: endMinute,
          tz: new Date().getTimezoneOffset(),
          udid: this.device.uuid
        }
      }).then(function successCallback(response) {
        console.log("PUSH setQuietTime " + JSON.stringify(response));
        return response.data;
      }, function errorCallback(response) {
        console.log("ERROR PUSH setQuietTime " + JSON.stringify(response));
        return {responseStatus: 'timeout', response: response }
      });

    },

    setTags: function(tags){
      return this._http({
        url: config.serverUrl()+"/device_tags/"+config.apiKey(),
        method: "POST",
        params: {
          tags: tags.toString(),
          udid: this.device.uuid
        }
      }).then(function successCallback(response) {
        console.log("PUSH setTags " + JSON.stringify(response));
        return response.data;
      }, function errorCallback(response) {
        console.log("ERROR PUSH setTags " + JSON.stringify(response));
        return {responseStatus: 'timeout', response: response }
      });

    },

    getSettings: function(){
      return this._http({
        url: config.serverUrl()+"/device_settings/"+config.apiKey(),
        method: "GET",
        params: {
          udid: this.device.uuid
        }
      }).then(function successCallback(response) {
        console.log("PUSH getSettings " + JSON.stringify(response));
        return response.data;
      }, function errorCallback(response) {
        console.log("ERROR PUSH getSettings " + JSON.stringify(response));
        return {responseStatus: 'timeout', response: response }
      });
    },
    logout: function(){
        //https://github.com/arnesson/cordova-plugin-firebase/issues/463
        //if (global.FirebasePlugin) global.FirebasePlugin.unregister();
    }

  };

  global.pushMeApp = pushMeApp;

})(window);








// usage

// setting alias
// pushMeApp.getAlias(loginData.email).then(function(result){
//   if (result.responseStatus != null){
//     console.log("Getting Alias error [" + result.responseStatus + "] " + result.responseMessage)
//   } else {
//     if (result.alias != loginData.email) {
//       pushMeApp.setAlias(loginData.email);
//     }
//   }
// })
